@@ -1,12 +1,40 @@
+/*
+ * rofi
+ *
+ * MIT/X11 License
+ * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+ * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+ * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+ * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #ifndef ROFI_VIEW_INTERNAL_H
 #define ROFI_VIEW_INTERNAL_H
+#include "widgets/container.h"
 #include "widgets/widget.h"
 #include "widgets/textbox.h"
-#include "widgets/separator.h"
 #include "widgets/listview.h"
 #include "widgets/box.h"
 #include "keyb.h"
-#include "x11-helper.h"
+#include "xcb.h"
+#include "theme.h"
 
 /**
  * @ingroup ViewHandle
@@ -22,24 +50,24 @@ struct RofiViewState
 
     /** Flag indicating if view needs to be refiltered. */
     int              refilter;
-
-    /** Main #box widget holding different elements. */
-    box              *main_box;
-    /** #box widget packing the input bar widgets. */
-    box              *input_bar;
+    /** Widget representing the main container. */
+    box              *main_window;
     /** #textbox showing the prompt in the input bar. */
     textbox          *prompt;
     /** #textbox with the user input in the input bar. */
     textbox          *text;
     /** #textbox showing the state of the case sensitive and sortng. */
     textbox          *case_indicator;
-    /** #separator widget below the input bar. */
-    separator        *input_bar_separator;
 
     /** #listview holding the displayed elements. */
     listview         *list_view;
     /** #textbox widget showing the overlay. */
     textbox          *overlay;
+    /** #container holding the message box */
+    container        *mesg_box;
+    /** #textbox containing the message entry */
+    textbox          *mesg_tb;
+
     /** Array with the levenshtein distance for each eleemnt. */
     int              *distance;
     /** Array with the translation between the filtered and unfiltered list. */
@@ -63,8 +91,6 @@ struct RofiViewState
     unsigned int     selected_line;
     /** The return state of the view */
     MenuReturn       retv;
-    /** Calculated border width */
-    unsigned int     border;
     /** Monitor #workarea the view is displayed on */
     workarea         mon;
 
@@ -92,8 +118,15 @@ struct RofiViewState
     /** Y position of the view */
     int              y;
 
+    struct
+    {
+        int    x;
+        int    y;
+        widget *motion_target;
+    }      mouse;
+
     /** Regexs used for matching */
-    GRegex           **tokens;
+    GRegex **tokens;
 };
 /** @} */
 #endif
