@@ -1,7 +1,33 @@
+/*
+ * rofi
+ *
+ * MIT/X11 License
+ * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+ * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+ * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+ * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #ifndef ROFI_VIEW_H
 #define ROFI_VIEW_H
-
-#include "xkb.h"
+#include "mode.h"
 
 /**
  * @defgroup View View
@@ -33,15 +59,14 @@ typedef enum
 /**
  * @param sw the Mode to show.
  * @param input A pointer to a string where the inputted data is placed.
- * @param message Extra message to display.
- * @param flags   Flags indicating state of the menu.
+ * @param menu_flags   Flags indicating state of the menu.
  * @param finalize the finailze callback
  *
  * Main menu callback.
  *
  * @returns The command issued (see MenuReturn)
  */
-RofiViewState *rofi_view_create ( Mode *sw, const char *input, const char *message, MenuFlags flags, void ( *finalize )( RofiViewState * ) );
+RofiViewState *rofi_view_create ( Mode *sw, const char *input, MenuFlags menu_flags, void ( *finalize )( RofiViewState * ) );
 
 /**
  * @param state The Menu Handle
@@ -68,12 +93,31 @@ MenuReturn rofi_view_get_return_value ( const RofiViewState *state );
 unsigned int rofi_view_get_next_position ( const RofiViewState *state );
 /**
  * @param state the Menu handle
- * @param event the event to handle
- * @param xkb  the keyboard handle
+ * @param text The text to add to the input box
  *
- * Process an Xevent.
+ * Update the state if needed.
  */
-void rofi_view_itterrate ( RofiViewState *state, xcb_generic_event_t *event, xkb_stuff *xkb );
+void rofi_view_handle_text ( RofiViewState *state, char *text );
+/**
+ * @param state the Menu handle
+ * @param x The X coordinates of the motion
+ * @param y The Y coordinates of the motion
+ *
+ * Update the state if needed.
+ */
+void rofi_view_handle_mouse_motion ( RofiViewState *state, gint x, gint y );
+/**
+ * @param state the Menu handle
+ *
+ * Update the state if needed.
+ */
+void rofi_view_maybe_update ( RofiViewState *state );
+void rofi_view_temp_configure_notify ( RofiViewState *state, xcb_configure_notify_event_t *xce );
+void rofi_view_temp_click_to_exit ( RofiViewState *state, xcb_window_t target );
+/**
+ * Update the state if needed.
+ */
+void rofi_view_frame_callback ( void );
 /**
  * @param state the Menu handle
  *
@@ -113,18 +157,12 @@ void rofi_view_restart ( RofiViewState *state );
 
 /**
  * @param state The handle to the view
- *
- * Update the state of the view. This involves filter state.
- */
-void rofi_view_update ( RofiViewState *state );
-
-/**
- * @param state The handle to the view
- * @param action The keyboard action
+ * @param scope The scope of the action
+ * @param action The action
  *
  * @returns TRUE if action was handled.
  */
-gboolean rofi_view_trigger_action ( RofiViewState *state, KeyBindingAction action );
+gboolean rofi_view_trigger_action ( RofiViewState *state, BindingsScope scope, guint action );
 
 /**
  * @param state The handle to the view
@@ -254,5 +292,17 @@ void rofi_view_workers_initialize ( void );
  * Stop all threads and free the resources used by the threadpool
  */
 void rofi_view_workers_finalize ( void );
+
+/**
+ * Return the current monitor workarea.
+ *
+ * @returns the current monitor workarea
+ */
+void rofi_view_get_current_monitor ( int *width, int *height );
+
+/**
+ * Takes a screenshot.
+ */
+void rofi_capture_screenshot ( void );
 /**@}*/
 #endif
