@@ -1,8 +1,8 @@
-/**
+/*
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2017 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -25,6 +25,8 @@
  *
  */
 
+#define G_LOG_DOMAIN    "Dialogs.DMenu"
+
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -47,8 +49,6 @@
 #include "xrmoptions.h"
 #include "view.h"
 
-#define LOG_DOMAIN    "Dialogs.DMenu"
-
 struct range_pair
 {
     unsigned int start;
@@ -105,7 +105,7 @@ typedef struct
 static void async_close_callback ( GObject *source_object, GAsyncResult *res, G_GNUC_UNUSED gpointer user_data )
 {
     g_input_stream_close_finish ( G_INPUT_STREAM ( source_object ), res, NULL );
-    g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Closing data stream." );
+    g_debug ( "Closing data stream." );
 }
 
 static void read_add ( DmenuModePrivateData * pd, char *data, gsize len )
@@ -115,7 +115,6 @@ static void read_add ( DmenuModePrivateData * pd, char *data, gsize len )
         pd->cmd_list             = g_realloc ( pd->cmd_list, ( pd->cmd_list_real_length ) * sizeof ( char* ) );
     }
     char *utfstr = rofi_force_utf8 ( data, len );
-
     pd->cmd_list[pd->cmd_list_length]     = utfstr;
     pd->cmd_list[pd->cmd_list_length + 1] = NULL;
 
@@ -137,25 +136,28 @@ static void async_read_callback ( GObject *source_object, GAsyncResult *res, gpo
         g_data_input_stream_read_upto_async ( pd->data_input_stream, &( pd->separator ), 1, G_PRIORITY_LOW, pd->cancel,
                                               async_read_callback, pd );
         return;
-    } else {
+    }
+    else {
         GError *error = NULL;
         // Absorb separator, already in buffer so should not block.
         // If error == NULL end of stream..
-        g_data_input_stream_read_byte ( stream, NULL, &error);
+        g_data_input_stream_read_byte ( stream, NULL, &error );
         if (  error == NULL ) {
             // Add empty line.
-            read_add ( pd, "", 0);
+            read_add ( pd, "", 0 );
             rofi_view_reload ();
 
             g_data_input_stream_read_upto_async ( pd->data_input_stream, &( pd->separator ), 1, G_PRIORITY_LOW, pd->cancel,
-                    async_read_callback, pd );
+                                                  async_read_callback, pd );
             return;
-        } else {
-            g_error_free (error);
+        }
+        else {
+            g_error_free ( error );
         }
     }
     if ( !g_cancellable_is_cancelled ( pd->cancel ) ) {
         // Hack, don't use get active.
+        g_debug ( "Clearing overlay" );
         rofi_view_set_overlay ( rofi_view_get_active (), NULL );
         g_input_stream_close_async ( G_INPUT_STREAM ( stream ), G_PRIORITY_LOW, pd->cancel, async_close_callback, pd );
     }
@@ -163,13 +165,25 @@ static void async_read_callback ( GObject *source_object, GAsyncResult *res, gpo
 
 static void async_read_cancel ( G_GNUC_UNUSED GCancellable *cancel, G_GNUC_UNUSED gpointer data )
 {
-    g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Cancelled the async read." );
+    g_debug ( "Cancelled the async read." );
 }
 
-static void get_dmenu_async ( DmenuModePrivateData *pd )
+static int get_dmenu_async ( DmenuModePrivateData *pd, int sync_pre_read )
 {
+    while ( sync_pre_read-- ) {
+        gsize len   = 0;
+        char  *data = g_data_input_stream_read_upto ( pd->data_input_stream, &( pd->separator ), 1, &len, NULL, NULL );
+        if ( data == NULL ) {
+            g_input_stream_close_async ( G_INPUT_STREAM ( pd->input_stream ), G_PRIORITY_LOW, pd->cancel, async_close_callback, pd );
+            return FALSE;
+        }
+        g_data_input_stream_read_byte ( pd->data_input_stream, NULL, NULL );
+        read_add ( pd, data, len );
+        g_free ( data );
+    }
     g_data_input_stream_read_upto_async ( pd->data_input_stream, &( pd->separator ), 1, G_PRIORITY_LOW, pd->cancel,
                                           async_read_callback, pd );
+    return TRUE;
 }
 static void get_dmenu_sync ( DmenuModePrivateData *pd )
 {
@@ -257,7 +271,7 @@ static gchar * dmenu_format_output_string ( const DmenuModePrivateData *pd, cons
     return retv ? retv : g_strdup ( "" );
 }
 
-static char *get_display_data ( const Mode *data, unsigned int index, int *state, int get_entry )
+static char *get_display_data ( const Mode *data, unsigned int index, int *state, G_GNUC_UNUSED GList **list, int get_entry )
 {
     Mode                 *sw    = (Mode *) data;
     DmenuModePrivateData *pd    = (DmenuModePrivateData *) mode_get_private_data ( sw );
@@ -429,7 +443,7 @@ static int dmenu_mode_init ( Mode *sw )
         char *estr = rofi_expand_path ( str );
         fd = open ( str, O_RDONLY );
         if ( fd < 0 ) {
-            char *msg = g_markup_printf_escaped ( "Failed to open file: <b>%s</b>:\n\t<i>%s</i>", estr, strerror ( errno ) );
+            char *msg = g_markup_printf_escaped ( "Failed to open file: <b>%s</b>:\n\t<i>%s</i>", estr, g_strerror ( errno ) );
             rofi_view_error_dialog ( msg, TRUE );
             g_free ( msg );
             g_free ( estr );
@@ -437,11 +451,13 @@ static int dmenu_mode_init ( Mode *sw )
         }
         g_free ( estr );
     }
-    pd->cancel            = g_cancellable_new ();
-    pd->cancel_source     = g_cancellable_connect ( pd->cancel, G_CALLBACK ( async_read_cancel ), pd, NULL );
-    pd->input_stream      = g_unix_input_stream_new ( fd, fd != STDIN_FILENO );
-    pd->data_input_stream = g_data_input_stream_new ( pd->input_stream );
-
+    // If input is stdin, and a tty, do not read as rofi grabs input and therefor blocks.
+    if ( !( fd == STDIN_FILENO && isatty ( fd ) == 1 ) ) {
+        pd->cancel            = g_cancellable_new ();
+        pd->cancel_source     = g_cancellable_connect ( pd->cancel, G_CALLBACK ( async_read_cancel ), pd, NULL );
+        pd->input_stream      = g_unix_input_stream_new ( fd, fd != STDIN_FILENO );
+        pd->data_input_stream = g_data_input_stream_new ( pd->input_stream );
+    }
     gchar *columns = NULL;
     if ( find_arg_str ( "-display-columns", &columns ) ) {
         pd->columns          = g_strsplit ( columns, ",", 0 );
@@ -454,7 +470,15 @@ static int dmenu_mode_init ( Mode *sw )
 static int dmenu_token_match ( const Mode *sw, GRegex **tokens, unsigned int index )
 {
     DmenuModePrivateData *rmpd = (DmenuModePrivateData *) mode_get_private_data ( sw );
-    return token_match ( tokens, rmpd->cmd_list[index] );
+    return helper_token_match ( tokens, rmpd->cmd_list[index] );
+}
+static char *dmenu_get_message ( const Mode *sw )
+{
+    DmenuModePrivateData *pd = (DmenuModePrivateData *) mode_get_private_data ( sw );
+    if ( pd->message ) {
+        return g_strdup ( pd->message );
+    }
+    return NULL;
 }
 
 #include "mode-private.h"
@@ -471,6 +495,7 @@ Mode dmenu_mode =
     ._get_display_value = get_display_data,
     ._get_completion    = NULL,
     ._preprocess_input  = NULL,
+    ._get_message       = dmenu_get_message,
     .private_data       = NULL,
     .free               = NULL,
     .display_name       = "dmenu:"
@@ -537,7 +562,25 @@ static void dmenu_finalize ( RofiViewState *state )
             restart = ( find_arg ( "-only-match" ) >= 0 );
         }
         else if ( pd->selected_line != UINT32_MAX ) {
-            if ( ( mretv & ( MENU_OK | MENU_QUICK_SWITCH ) ) && cmd_list[pd->selected_line] != NULL ) {
+            if ( ( mretv & MENU_CUSTOM_ACTION ) && pd->multi_select ) {
+                restart = TRUE;
+                if ( pd->selected_list == NULL ) {
+                    pd->selected_list = g_malloc0 ( sizeof ( uint32_t ) * ( pd->cmd_list_length / 32 + 1 ) );
+                }
+                pd->selected_count += ( bitget ( pd->selected_list, pd->selected_line ) ? ( -1 ) : ( 1 ) );
+                bittoggle ( pd->selected_list, pd->selected_line );
+                // Move to next line.
+                pd->selected_line = MIN ( next_pos, cmd_list_length - 1 );
+                if ( pd->selected_count > 0 ) {
+                    char *str = g_strdup_printf ( "%u/%u", pd->selected_count, pd->cmd_list_length );
+                    rofi_view_set_overlay ( state, str );
+                    g_free ( str );
+                }
+                else {
+                    rofi_view_set_overlay ( state, NULL );
+                }
+            }
+            else if ( ( mretv & ( MENU_OK | MENU_QUICK_SWITCH ) ) && cmd_list[pd->selected_line] != NULL ) {
                 dmenu_print_results ( pd, input );
                 retv = TRUE;
                 if ( ( mretv & MENU_QUICK_SWITCH ) ) {
@@ -547,7 +590,9 @@ static void dmenu_finalize ( RofiViewState *state )
                 dmenu_finish ( state, retv );
                 return;
             }
-            pd->selected_line = next_pos - 1;
+            else {
+                pd->selected_line = next_pos - 1;
+            }
         }
         // Restart
         rofi_view_restart ( state );
@@ -613,17 +658,23 @@ int dmenu_switcher_dialog ( void )
     MenuFlags            menu_flags = MENU_NORMAL;
     DmenuModePrivateData *pd        = (DmenuModePrivateData *) dmenu_mode.private_data;
     int                  async      = TRUE;
+
     // For now these only work in sync mode.
     if ( find_arg ( "-sync" ) >= 0 || find_arg ( "-dump" ) >= 0 || find_arg ( "-select" ) >= 0
          || find_arg ( "-no-custom" ) >= 0 || find_arg ( "-only-match" ) >= 0 || config.auto_select ||
          find_arg ( "-selected-row" ) >= 0 ) {
         async = FALSE;
     }
-    if ( async ) {
-        get_dmenu_async ( pd );
-    }
-    else {
-        get_dmenu_sync ( pd );
+    // Check if the subsystem is setup for reading, otherwise do not read.
+    if ( pd->cancel != NULL ) {
+        if ( async ) {
+            unsigned int pre_read = 25;
+            find_arg_uint ( "-async-pre-read", &pre_read );
+            async = get_dmenu_async ( pd, pre_read );
+        }
+        else {
+            get_dmenu_sync ( pd );
+        }
     }
     char         *input          = NULL;
     unsigned int cmd_list_length = pd->cmd_list_length;
@@ -660,7 +711,7 @@ int dmenu_switcher_dialog ( void )
         GRegex       **tokens = tokenize ( select, config.case_sensitive );
         unsigned int i        = 0;
         for ( i = 0; i < cmd_list_length; i++ ) {
-            if ( token_match ( tokens, cmd_list[i] ) ) {
+            if ( helper_token_match ( tokens, cmd_list[i] ) ) {
                 pd->selected_line = i;
                 break;
             }
@@ -671,7 +722,7 @@ int dmenu_switcher_dialog ( void )
         GRegex       **tokens = tokenize ( config.filter ? config.filter : "", config.case_sensitive );
         unsigned int i        = 0;
         for ( i = 0; i < cmd_list_length; i++ ) {
-            if ( tokens == NULL || token_match ( tokens, cmd_list[i] ) ) {
+            if ( tokens == NULL || helper_token_match ( tokens, cmd_list[i] ) ) {
                 dmenu_output_formatted_line ( pd->format, cmd_list[i], i, config.filter );
             }
         }
@@ -681,9 +732,9 @@ int dmenu_switcher_dialog ( void )
         return TRUE;
     }
     find_arg_str (  "-p", &( dmenu_mode.display_name ) );
-    RofiViewState *state = rofi_view_create ( &dmenu_mode, input, pd->message, menu_flags, dmenu_finalize );
+    RofiViewState *state = rofi_view_create ( &dmenu_mode, input, menu_flags, dmenu_finalize );
     // @TODO we should do this better.
-    if ( async ) {
+    if ( async && ( pd->cancel != NULL ) ) {
         rofi_view_set_overlay ( state, "Loading.. " );
     }
     rofi_view_set_selected_line ( state, pd->selected_line );
@@ -711,4 +762,6 @@ void print_dmenu_options ( void )
     print_help_msg ( "-sep", "[char]", "Element separator.", "'\\n'", is_term );
     print_help_msg ( "-input", "[filename]", "Read input from file instead from standard input.", NULL, is_term );
     print_help_msg ( "-sync", "", "Force dmenu to first read all input data, then show dialog.", NULL, is_term );
+    print_help_msg ( "-async-pre-read", "[number]", "Read several entries blocking before switching to async mode", "25", is_term );
+    print_help_msg ( "-w", "windowid", "Position over window with X11 windowid.", NULL, is_term );
 }
