@@ -1,8 +1,8 @@
-/**
+/*
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2017 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -24,6 +24,7 @@
  * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  */
+
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -93,7 +94,7 @@ static void help_keys_mode_destroy ( Mode *sw )
     }
 }
 
-static char *_get_display_value ( const Mode *sw, unsigned int selected_line, int *state, int get_entry )
+static char *_get_display_value ( const Mode *sw, unsigned int selected_line, int *state, G_GNUC_UNUSED GList **list, int get_entry )
 {
     KeysHelpModePrivateData *pd = (KeysHelpModePrivateData *) mode_get_private_data ( sw );
     *state |= MARKUP;
@@ -108,7 +109,7 @@ static int help_keys_token_match ( const Mode *data,
                                    )
 {
     KeysHelpModePrivateData *rmpd = (KeysHelpModePrivateData *) mode_get_private_data ( data );
-    return token_match ( tokens, rmpd->messages[index] );
+    return helper_token_match ( tokens, rmpd->messages[index] );
 }
 
 static unsigned int help_keys_mode_get_num_entries ( const Mode *sw )
