@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2017 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -29,6 +29,10 @@
  * \ingroup RUNMode
  * @{
  */
+
+/** The log domain of this dialog. */
+#define G_LOG_DOMAIN    "Dialogs.Run"
+
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -49,14 +53,13 @@
 #include "dialogs/run.h"
 
 #include "mode-private.h"
+
+#include "timings.h"
 /**
  * Name of the history file where previously choosen commands are stored.
  */
 #define RUN_CACHE_FILE    "rofi-3.runcache"
 
-/** The log domain of this dialog. */
-#define LOG_DOMAIN        "Dialogs.Run"
-
 /**
  * The internal data structure holding the private data of the Run Mode.
  */
@@ -68,41 +71,6 @@ typedef struct
     unsigned int cmd_list_length;
 } RunModePrivateData;
 
-/**
- * @param cmd The cmd to execute
- * @param run_in_term Indicate if command should be run in a terminal
- *
- * Execute command.
- *
- * @returns FALSE On failure, TRUE on success
- */
-static inline int execsh ( const char *cmd, int run_in_term )
-{
-    int  retv   = TRUE;
-    char **args = NULL;
-    int  argc   = 0;
-    if ( run_in_term ) {
-        helper_parse_setup ( config.run_shell_command, &args, &argc, "{cmd}", cmd, NULL );
-    }
-    else {
-        helper_parse_setup ( config.run_command, &args, &argc, "{cmd}", cmd, NULL );
-    }
-    GError *error = NULL;
-    g_spawn_async ( NULL, args, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error );
-    if ( error != NULL ) {
-        char *msg = g_strdup_printf ( "Failed to execute: '%s'\nError: '%s'", cmd, error->message );
-        rofi_view_error_dialog ( msg, FALSE  );
-        g_free ( msg );
-        // print error.
-        g_error_free ( error );
-        retv = FALSE;
-    }
-
-    // Free the args list.
-    g_strfreev ( args );
-    return retv;
-}
-
 /**
  * @param cmd The cmd to execute
  * @param run_in_term Indicate if command should be run in a terminal
@@ -118,13 +86,15 @@ static void exec_cmd ( const char *cmd, int run_in_term )
     gsize lf_cmd_size = 0;
     gchar *lf_cmd     = g_locale_from_utf8 ( cmd, -1, NULL, &lf_cmd_size, &error );
     if ( error != NULL ) {
-        fprintf ( stderr, "Failed to convert command to locale encoding: %s\n", error->message );
+        g_warning ( "Failed to convert command to locale encoding: %s", error->message );
         g_error_free ( error );
         return;
     }
 
-    char *path = g_build_filename ( cache_dir, RUN_CACHE_FILE, NULL );
-    if (  execsh ( lf_cmd, run_in_term ) ) {
+    char                     *path   = g_build_filename ( cache_dir, RUN_CACHE_FILE, NULL );
+    RofiHelperExecuteContext context = { .name = NULL };
+    // FIXME: assume startup notification support for terminals
+    if (  helper_execute_command ( NULL, lf_cmd, run_in_term, run_in_term ? &context : NULL ) ) {
         /**
          * This happens in non-critical time (After launching app)
          * It is allowed to be a bit slower.
@@ -176,7 +146,7 @@ static int sort_func ( const void *a, const void *b, G_GNUC_UNUSED void *data )
     else if ( bstr == NULL ) {
         return -1;
     }
-    return g_ascii_strcasecmp ( astr, bstr );
+    return g_strcmp0 ( astr, bstr );
 }
 
 /**
@@ -220,8 +190,8 @@ static char ** get_apps_external ( char **retv, unsigned int *length, unsigned i
                 free ( buffer );
             }
             if ( fclose ( inp ) != 0 ) {
-                fprintf ( stderr, "Failed to close stdout off executor script: '%s'\n",
-                          strerror ( errno ) );
+                g_warning ( "Failed to close stdout off executor script: '%s'",
+                            g_strerror ( errno ) );
             }
         }
     }
@@ -254,7 +224,7 @@ static char ** get_apps ( unsigned int *length )
     gsize l        = 0;
     gchar *homedir = g_locale_to_utf8 (  g_get_home_dir (), -1, NULL, &l, &error );
     if ( error != NULL ) {
-        g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Failed to convert homedir to UTF-8: %s", error->message );
+        g_debug ( "Failed to convert homedir to UTF-8: %s", error->message );
         g_clear_error ( &error );
         g_free ( homedir );
         return NULL;
@@ -263,15 +233,17 @@ static char ** get_apps ( unsigned int *length )
     const char *const sep                 = ":";
     char              *strtok_savepointer = NULL;
     for ( const char *dirname = strtok_r ( path, sep, &strtok_savepointer ); dirname != NULL; dirname = strtok_r ( NULL, sep, &strtok_savepointer ) ) {
-        DIR *dir = opendir ( dirname );
+        char *fpath = rofi_expand_path ( dirname );
+        DIR  *dir   = opendir ( fpath );
+        g_debug ( "Checking path %s for executable.", fpath );
+        g_free ( fpath );
 
-        g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Checking path %s for executable.", dirname );
         if ( dir != NULL ) {
             struct dirent *dent;
             gsize         dirn_len = 0;
             gchar         *dirn    = g_locale_to_utf8 ( dirname, -1, NULL, &dirn_len, &error );
             if ( error != NULL ) {
-                g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Failed to convert directory name to UTF-8: %s", error->message );
+                g_debug ( "Failed to convert directory name to UTF-8: %s", error->message );
                 g_clear_error ( &error );
                 closedir ( dir );
                 continue;
@@ -299,7 +271,7 @@ static char ** get_apps ( unsigned int *length )
                 gsize name_len;
                 gchar *name = g_filename_to_utf8 ( dent->d_name, -1, NULL, &name_len, &error );
                 if ( error != NULL ) {
-                    g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Failed to convert filename to UTF-8: %s", error->message );
+                    g_debug ( "Failed to convert filename to UTF-8: %s", error->message );
                     g_clear_error ( &error );
                     g_free ( name );
                     continue;
@@ -423,7 +395,7 @@ static ModeMode run_mode_result ( Mode *sw, int mretv, char **input, unsigned in
     return retv;
 }
 
-static char *_get_display_value ( const Mode *sw, unsigned int selected_line, G_GNUC_UNUSED int *state, int get_entry )
+static char *_get_display_value ( const Mode *sw, unsigned int selected_line, G_GNUC_UNUSED int *state, G_GNUC_UNUSED GList **list, int get_entry )
 {
     const RunModePrivateData *rmpd = (const RunModePrivateData *) sw->private_data;
     return get_entry ? g_strdup ( rmpd->cmd_list[selected_line] ) : NULL;
@@ -431,7 +403,7 @@ static char *_get_display_value ( const Mode *sw, unsigned int selected_line, G_
 static int run_token_match ( const Mode *sw, GRegex **tokens, unsigned int index )
 {
     const RunModePrivateData *rmpd = (const RunModePrivateData *) sw->private_data;
-    return token_match ( tokens, rmpd->cmd_list[index] );
+    return helper_token_match ( tokens, rmpd->cmd_list[index] );
 }
 
 #include "mode-private.h"
