@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2017 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -29,7 +29,14 @@
  * \ingroup SSHMode
  * @{
  */
+
+/**
+ * Log domain for the ssh modi.
+ */
+#define G_LOG_DOMAIN    "Dialogs.Ssh"
+
 #include <config.h>
+#include <glib.h>
 #include <stdlib.h>
 #include <stdio.h>
 
@@ -42,6 +49,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <helper.h>
+#include <glob.h>
 
 #include "rofi.h"
 #include "settings.h"
@@ -66,29 +74,24 @@
  *
  * @returns FALSE On failure, TRUE on success
  */
-static inline int execshssh ( const char *host )
+static int execshssh ( const char *host )
 {
     char **args = NULL;
     int  argsv  = 0;
-    helper_parse_setup ( config.ssh_command, &args, &argsv, "{host}", host, NULL );
 
-    GError *error = NULL;
-    g_spawn_async ( NULL, args, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error );
+    helper_parse_setup ( config.ssh_command, &args, &argsv, "{host}", host, (char *) 0 );
 
-    if ( error != NULL ) {
-        char *msg = g_strdup_printf ( "Failed to execute: 'ssh %s'\nError: '%s'", host, error->message );
-        rofi_view_error_dialog ( msg, FALSE  );
-        g_free ( msg );
-        // print error.
-        g_error_free ( error );
+    gsize l     = strlen ( "Connecting to '' via rofi" ) + strlen ( host ) + 1;
+    gchar *desc = g_newa ( gchar, l );
 
-        g_strfreev ( args );
-        return FALSE;
-    }
-    // Free the args list.
-    g_strfreev ( args );
+    g_snprintf ( desc, l, "Connecting to '%s' via rofi", host );
 
-    return TRUE;
+    RofiHelperExecuteContext context = {
+        .name        = "ssh",
+        .description = desc,
+        .command     = "ssh",
+    };
+    return helper_execute ( NULL, args, "ssh ", host, &context );
 }
 
 /**
@@ -138,7 +141,7 @@ static void delete_ssh ( const char *host )
  */
 static char **read_known_hosts_file ( char ** retv, unsigned int *length )
 {
-    char *path = g_build_filename ( g_getenv ( "HOME" ), ".ssh", "known_hosts", NULL );
+    char *path = g_build_filename ( g_get_home_dir (), ".ssh", "known_hosts", NULL );
     FILE *fd   = fopen ( path, "r" );
     if ( fd != NULL ) {
         char   *buffer       = NULL;
@@ -172,7 +175,7 @@ static char **read_known_hosts_file ( char ** retv, unsigned int *length )
             free ( buffer );
         }
         if ( fclose ( fd ) != 0 ) {
-            fprintf ( stderr, "Failed to close hosts file: '%s'\n", strerror ( errno ) );
+            g_warning ( "Failed to close hosts file: '%s'", g_strerror ( errno ) );
         }
     }
 
@@ -198,7 +201,7 @@ static char **read_hosts_file ( char ** retv, unsigned int *length )
         // Reading one line per time.
         while ( getline ( &buffer, &buffer_length, fd ) > 0 ) {
             // Evaluate one line.
-            unsigned int index  = 0, ti = 0;
+            unsigned int index = 0, ti = 0;
             char         *token = buffer;
 
             // Tokenize it.
@@ -247,13 +250,112 @@ static char **read_hosts_file ( char ** retv, unsigned int *length )
             free ( buffer );
         }
         if ( fclose ( fd ) != 0 ) {
-            fprintf ( stderr, "Failed to close hosts file: '%s'\n", strerror ( errno ) );
+            g_warning ( "Failed to close hosts file: '%s'", g_strerror ( errno ) );
         }
     }
 
     return retv;
 }
 
+static void parse_ssh_config_file ( const char *filename, char ***retv, unsigned int *length, unsigned int num_favorites )
+{
+    FILE *fd = fopen ( filename, "r" );
+
+    g_debug ( "Parsing ssh config file: %s", filename );
+    if ( fd != NULL ) {
+        char   *buffer         = NULL;
+        size_t buffer_length   = 0;
+        char   *strtok_pointer = NULL;
+        while ( getline ( &buffer, &buffer_length, fd ) > 0 ) {
+            // Each line is either empty, a comment line starting with a '#'
+            // character or of the form "keyword [=] arguments", where there may
+            // be multiple (possibly quoted) arguments separated by whitespace.
+            // The keyword is separated from its arguments by whitespace OR by
+            // optional whitespace and a '=' character.
+            char *token = strtok_r ( buffer, SSH_TOKEN_DELIM, &strtok_pointer );
+
+            // Skip empty lines and comment lines. Also skip lines where the
+            // keyword is not "Host".
+            if ( !token || *token == '#' ) {
+                continue;
+            }
+
+            if ( g_strcmp0 ( token, "Include" ) == 0 ) {
+                token = strtok_r ( NULL, SSH_TOKEN_DELIM, &strtok_pointer );
+                g_debug ( "Found Include: %s", token );
+                gchar *path      = rofi_expand_path ( token );
+                gchar *full_path = NULL;
+                if ( !g_path_is_absolute ( path ) ) {
+                    char *dirname = g_path_get_dirname ( filename );
+                    full_path = g_build_filename ( dirname, path, NULL );
+                    g_free ( dirname );
+                }
+                else {
+                    full_path = g_strdup ( path );
+                }
+                glob_t globbuf = { .gl_pathc = 0, .gl_pathv = NULL, .gl_offs = 0 };
+
+                if ( glob ( full_path, 0, NULL, &globbuf ) == 0 ) {
+                    for ( size_t iter = 0; iter < globbuf.gl_pathc; iter++ ) {
+                        parse_ssh_config_file ( globbuf.gl_pathv[iter], retv, length, num_favorites );
+                    }
+                }
+                globfree ( &globbuf );
+
+                g_free ( full_path );
+                g_free ( path );
+            }
+            else if ( g_strcmp0 ( token, "Host" ) == 0 ) {
+                // Now we know that this is a "Host" line.
+                // The "Host" keyword is followed by one more host names separated
+                // by whitespace; while host names may be quoted with double quotes
+                // to represent host names containing spaces, we don't support this
+                // (how many host names contain spaces?).
+                while ( ( token = strtok_r ( NULL, SSH_TOKEN_DELIM, &strtok_pointer ) ) ) {
+                    // We do not want to show wildcard entries, as you cannot ssh to them.
+                    const char *const sep = "*?";
+                    if ( *token == '!' || strpbrk ( token, sep ) ) {
+                        continue;
+                    }
+
+                    // If comment, skip from now on.
+                    if ( *token == '#' ) {
+                        break;
+                    }
+
+                    // Is this host name already in the history file?
+                    // This is a nice little penalty, but doable? time will tell.
+                    // given num_favorites is max 25.
+                    int found = 0;
+                    for ( unsigned int j = 0; j < num_favorites; j++ ) {
+                        if ( !g_ascii_strcasecmp ( token, ( *retv )[j] ) ) {
+                            found = 1;
+                            break;
+                        }
+                    }
+
+                    if ( found ) {
+                        continue;
+                    }
+
+                    // Add this host name to the list.
+                    ( *retv )                  = g_realloc ( ( *retv ), ( ( *length ) + 2 ) * sizeof ( char* ) );
+                    ( *retv )[( *length )]     = g_strdup ( token );
+                    ( *retv )[( *length ) + 1] = NULL;
+                    ( *length )++;
+                }
+            }
+        }
+        if ( buffer != NULL ) {
+            free ( buffer );
+        }
+
+        if ( fclose ( fd ) != 0 ) {
+            g_warning ( "Failed to close ssh configuration file: '%s'", g_strerror ( errno ) );
+        }
+    }
+}
+
 /**
  * @param length The number of found ssh hosts [out]
  *
@@ -267,7 +369,7 @@ static char ** get_ssh (  unsigned int *length )
     unsigned int num_favorites = 0;
     char         *path;
 
-    if ( getenv ( "HOME" ) == NULL ) {
+    if ( g_get_home_dir () == NULL ) {
         return NULL;
     }
 
@@ -283,77 +385,10 @@ static char ** get_ssh (  unsigned int *length )
         retv = read_hosts_file ( retv, length );
     }
 
-    FILE       *fd = NULL;
-    const char *hd = getenv ( "HOME" );
+    const char *hd = g_get_home_dir ();
     path = g_build_filename ( hd, ".ssh", "config", NULL );
-    fd   = fopen ( path, "r" );
-
-    if ( fd != NULL ) {
-        char   *buffer         = NULL;
-        size_t buffer_length   = 0;
-        char   *strtok_pointer = NULL;
-        while ( getline ( &buffer, &buffer_length, fd ) > 0 ) {
-            // Each line is either empty, a comment line starting with a '#'
-            // character or of the form "keyword [=] arguments", where there may
-            // be multiple (possibly quoted) arguments separated by whitespace.
-            // The keyword is separated from its arguments by whitespace OR by
-            // optional whitespace and a '=' character.
-            char *token = strtok_r ( buffer, SSH_TOKEN_DELIM, &strtok_pointer );
-
-            // Skip empty lines and comment lines. Also skip lines where the
-            // keyword is not "Host".
-            if ( !token || *token == '#' || g_ascii_strcasecmp ( token, "Host" ) ) {
-                continue;
-            }
-
-            // Now we know that this is a "Host" line.
-            // The "Host" keyword is followed by one more host names separated
-            // by whitespace; while host names may be quoted with double quotes
-            // to represent host names containing spaces, we don't support this
-            // (how many host names contain spaces?).
-            while ( ( token = strtok_r ( NULL, SSH_TOKEN_DELIM, &strtok_pointer ) ) ) {
-                // We do not want to show wildcard entries, as you cannot ssh to them.
-                const char *const sep = "*?";
-                if ( *token == '!' || strpbrk ( token, sep ) ) {
-                    continue;
-                }
-
-                // If comment, skip from now on.
-                if ( *token == '#' ) {
-                    break;
-                }
-
-                // Is this host name already in the history file?
-                // This is a nice little penalty, but doable? time will tell.
-                // given num_favorites is max 25.
-                int found = 0;
-                for ( unsigned int j = 0; j < num_favorites; j++ ) {
-                    if ( !g_ascii_strcasecmp ( token, retv[j] ) ) {
-                        found = 1;
-                        break;
-                    }
-                }
-
-                if ( found ) {
-                    continue;
-                }
-
-                // Add this host name to the list.
-                retv                  = g_realloc ( retv, ( ( *length ) + 2 ) * sizeof ( char* ) );
-                retv[( *length )]     = g_strdup ( token );
-                retv[( *length ) + 1] = NULL;
-                ( *length )++;
-            }
-        }
-        if ( buffer != NULL ) {
-            free ( buffer );
-        }
-
-        if ( fclose ( fd ) != 0 ) {
-            fprintf ( stderr, "Failed to close ssh configuration file: '%s'\n", strerror ( errno ) );
-        }
-    }
 
+    parse_ssh_config_file ( path, &retv, length, num_favorites );
     g_free ( path );
 
     return retv;
@@ -458,6 +493,7 @@ static void ssh_mode_destroy ( Mode *sw )
  * @param sw Object handle to the SSH Mode object
  * @param selected_line The line to view
  * @param state The state of the entry [out]
+ * @param attr_list List of extra rendering attributes to set [out]
  * @param get_entry
  *
  * Gets the string as it should be displayed and the display state.
@@ -465,7 +501,7 @@ static void ssh_mode_destroy ( Mode *sw )
  *
  * @return the string as it should be displayed and the display state.
  */
-static char *_get_display_value ( const Mode *sw, unsigned int selected_line, G_GNUC_UNUSED int *state, int get_entry )
+static char *_get_display_value ( const Mode *sw, unsigned int selected_line, G_GNUC_UNUSED int *state, G_GNUC_UNUSED GList **attr_list, int get_entry )
 {
     SSHModePrivateData *rmpd = (SSHModePrivateData *) mode_get_private_data ( sw );
     return get_entry ? g_strdup ( rmpd->hosts_list[selected_line] ) : NULL;
@@ -483,7 +519,7 @@ static char *_get_display_value ( const Mode *sw, unsigned int selected_line, G_
 static int ssh_token_match ( const Mode *sw, GRegex **tokens, unsigned int index )
 {
     SSHModePrivateData *rmpd = (SSHModePrivateData *) mode_get_private_data ( sw );
-    return token_match ( tokens, rmpd->hosts_list[index] );
+    return helper_token_match ( tokens, rmpd->hosts_list[index] );
 }
 #include "mode-private.h"
 Mode ssh_mode =
