@@ -1,8 +1,8 @@
-/**
+/*
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2017 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -24,6 +24,7 @@
  * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  */
+
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -145,7 +146,7 @@ void history_set ( const char *filename, const char *entry )
         list = __history_get_element_list ( fd, &length );
         // Close file, if fails let user know on stderr.
         if ( fclose ( fd ) != 0 ) {
-            fprintf ( stderr, "Failed to close history file: %s\n", strerror ( errno ) );
+            g_warning ( "Failed to close history file: %s", g_strerror ( errno ) );
         }
     }
     // Look if the entry exists.
@@ -178,14 +179,14 @@ void history_set ( const char *filename, const char *entry )
 
     fd = fopen ( filename, "w" );
     if ( fd == NULL ) {
-        fprintf ( stderr, "Failed to open file: %s\n", strerror ( errno ) );
+        g_warning ( "Failed to open file: %s", g_strerror ( errno ) );
     }
     else {
         // Write list.
         __history_write_element_list ( fd, list, length );
         // Close file, if fails let user know on stderr.
         if ( fclose ( fd ) != 0 ) {
-            fprintf ( stderr, "Failed to close history file: %s\n", strerror ( errno ) );
+            g_warning ( "Failed to close history file: %s", g_strerror ( errno ) );
         }
     }
     // Free the list.
@@ -208,7 +209,7 @@ void history_remove ( const char *filename, const char *entry )
     // Open file for reading and writing.
     FILE         *fd = g_fopen ( filename, "r" );
     if ( fd == NULL ) {
-        fprintf ( stderr, "Failed to open file: %s\n", strerror ( errno ) );
+        g_warning ( "Failed to open file: %s", g_strerror ( errno ) );
         return;
     }
     // Get list.
@@ -216,7 +217,7 @@ void history_remove ( const char *filename, const char *entry )
 
     // Close file, if fails let user know on stderr.
     if ( fclose ( fd ) != 0 ) {
-        fprintf ( stderr, "Failed to close history file: %s\n", strerror ( errno ) );
+        g_warning ( "Failed to close history file: %s", g_strerror ( errno ) );
     }
     // Find entry.
     for ( unsigned int iter = 0; !found && iter < length; iter++ ) {
@@ -244,11 +245,11 @@ void history_remove ( const char *filename, const char *entry )
             __history_write_element_list ( fd, list, length );
             // Close file, if fails let user know on stderr.
             if ( fclose ( fd ) != 0 ) {
-                fprintf ( stderr, "Failed to close history file: %s\n", strerror ( errno ) );
+                g_warning ( "Failed to close history file: %s", g_strerror ( errno ) );
             }
         }
         else{
-            fprintf ( stderr, "Failed to open file: %s\n", strerror ( errno ) );
+            g_warning ( "Failed to open file: %s", g_strerror ( errno ) );
         }
     }
 
@@ -277,7 +278,7 @@ char ** history_get_list ( const char *filename, unsigned int *length )
         // File that does not exists is not an error, so ignore it.
         // Everything else? panic.
         if ( errno != ENOENT ) {
-            fprintf ( stderr, "Failed to open file: %s\n", strerror ( errno ) );
+            g_warning ( "Failed to open file: %s", g_strerror ( errno ) );
         }
         return NULL;
     }
@@ -298,7 +299,7 @@ char ** history_get_list ( const char *filename, unsigned int *length )
 
     // Close file, if fails let user know on stderr.
     if ( fclose ( fd ) != 0 ) {
-        fprintf ( stderr, "Failed to close history file: %s\n", strerror ( errno ) );
+        g_warning ( "Failed to close history file: %s", g_strerror ( errno ) );
     }
     return retv;
 }
