@@ -1,9 +1,35 @@
+/*
+ * rofi
+ *
+ * MIT/X11 License
+ * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+ * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+ * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+ * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #include <stdio.h>
 #include <glib.h>
 #include <string.h>
 #include "rofi.h"
 #include "xrmoptions.h"
-#include "x11-helper.h"
 #include "mode.h"
 
 // This one should only be in mode implementations.
@@ -17,8 +43,7 @@ int mode_init ( Mode *mode )
 {
     g_return_val_if_fail ( mode != NULL, FALSE );
     g_return_val_if_fail ( mode->_init != NULL, FALSE );
-    mode->_init ( mode );
-    return TRUE;
+    return mode->_init ( mode );
 }
 
 void mode_destroy ( Mode *mode )
@@ -35,13 +60,25 @@ unsigned int mode_get_num_entries ( const Mode *mode )
     return mode->_get_num_entries ( mode );
 }
 
-char * mode_get_display_value ( const Mode *mode, unsigned int selected_line, int *state, int get_entry )
+char * mode_get_display_value ( const Mode *mode, unsigned int selected_line, int *state, GList **attribute_list, int get_entry )
 {
     g_assert ( mode != NULL );
     g_assert ( state != NULL );
     g_assert ( mode->_get_display_value != NULL );
 
-    return mode->_get_display_value ( mode, selected_line, state, get_entry );
+    return mode->_get_display_value ( mode, selected_line, state, attribute_list, get_entry );
+}
+
+cairo_surface_t * mode_get_icon ( const Mode *mode, unsigned int selected_line, int height )
+{
+    g_assert ( mode != NULL );
+
+    if ( mode->_get_icon != NULL ) {
+        return mode->_get_icon ( mode, selected_line, height );
+    }
+    else {
+        return NULL;
+    }
 }
 
 char * mode_get_completion ( const Mode *mode, unsigned int selected_line )
@@ -53,7 +90,7 @@ char * mode_get_completion ( const Mode *mode, unsigned int selected_line )
     else {
         int state;
         g_assert ( mode->_get_display_value != NULL );
-        return mode->_get_display_value ( mode, selected_line, &state, TRUE );
+        return mode->_get_display_value ( mode, selected_line, &state, NULL, TRUE );
     }
 }
 
@@ -124,4 +161,11 @@ char * mode_preprocess_input ( Mode *mode, const char *input )
     }
     return g_strdup ( input );
 }
+char *mode_get_message ( const Mode *mode )
+{
+    if ( mode->_get_message ) {
+        return mode->_get_message ( mode );
+    }
+    return NULL;
+}
 /*@}*/
