@@ -1,8 +1,8 @@
-/**
+/*
  * rofi
  *
  * MIT/X11 License
- * Modified 2016-2017 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -22,33 +22,118 @@
  * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
  * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
  * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
  */
 
+#define G_LOG_DOMAIN    "Widgets.Box"
+
 #include <config.h>
 #include <stdio.h>
 #include "widgets/widget.h"
 #include "widgets/widget-internal.h"
 #include "widgets/box.h"
+#include "theme.h"
 
-#define LOG_DOMAIN    "Widgets.Box"
+/** Default spacing used in the box*/
+#define DEFAULT_SPACING    2
 
 struct _box
 {
-    widget  widget;
-    boxType type;
-    int     max_size;
-    // Padding between elements
-    int     padding;
+    widget          widget;
+    RofiOrientation type;
+    int             max_size;
+    // RofiPadding between elements
+    RofiDistance    spacing;
 
-    GList   *children;
+    GList           *children;
 };
 
 static void box_update ( widget *wid  );
 
+static int box_get_desired_width  ( widget *wid )
+{
+    box *b      = (box *) wid;
+    int spacing = distance_get_pixel ( b->spacing, b->type );
+    int width   = 0;
+    if ( b->type == ROFI_ORIENTATION_HORIZONTAL ) {
+        int active_widgets = 0;
+        for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
+            widget * child = (widget *) iter->data;
+            if ( !child->enabled ) {
+                continue;
+            }
+            active_widgets++;
+            if ( child->expand == TRUE ) {
+                width += widget_get_desired_width ( child );
+                continue;
+            }
+            width += widget_get_desired_width ( child );
+        }
+        if ( active_widgets > 0 ) {
+            width += ( active_widgets - 1 ) * spacing;
+        }
+    }
+    else {
+        for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
+            widget * child = (widget *) iter->data;
+            if ( !child->enabled ) {
+                continue;
+            }
+            width = MAX ( widget_get_desired_width ( child ), width );
+        }
+    }
+    width += widget_padding_get_padding_width ( wid );
+    return width;
+}
+static int box_get_desired_height ( widget *wid )
+{
+    box *b      = (box *) wid;
+    int spacing = distance_get_pixel ( b->spacing, b->type );
+    int height  = 0;
+    if ( b->type == ROFI_ORIENTATION_VERTICAL ) {
+        int active_widgets = 0;
+        for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
+            widget * child = (widget *) iter->data;
+            if ( !child->enabled ) {
+                continue;
+            }
+            active_widgets++;
+            if ( child->expand == TRUE ) {
+                height += widget_get_desired_height ( child );
+                continue;
+            }
+            height += widget_get_desired_height ( child );
+        }
+        if ( active_widgets > 0 ) {
+            height += ( active_widgets - 1 ) * spacing;
+        }
+    }
+    else {
+        for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
+            widget * child = (widget *) iter->data;
+            if ( !child->enabled ) {
+                continue;
+            }
+            height = MAX ( widget_get_desired_height ( child ), height );
+        }
+    }
+    height += widget_padding_get_padding_height ( wid );
+    return height;
+}
+
 static void vert_calculate_size ( box *b )
 {
+    int spacing           = distance_get_pixel ( b->spacing, ROFI_ORIENTATION_VERTICAL );
     int expanding_widgets = 0;
     int active_widgets    = 0;
+    int rem_width         = widget_padding_get_remaining_width ( WIDGET ( b ) );
+    int rem_height        = widget_padding_get_remaining_height ( WIDGET ( b ) );
+    for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
+        widget * child = (widget *) iter->data;
+        if ( child->enabled && child->expand == FALSE ) {
+            widget_resize ( child, rem_width, widget_get_desired_height ( child ) );
+        }
+    }
     b->max_size = 0;
     for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
         widget * child = (widget *) iter->data;
@@ -60,18 +145,22 @@ static void vert_calculate_size ( box *b )
             expanding_widgets++;
             continue;
         }
-        b->max_size += child->h;
+        if ( child->h > 0 ) {
+            b->max_size += child->h;
+        }
     }
-    b->max_size += MAX ( 0, ( ( active_widgets - 1 ) * b->padding ) );
-    if ( b->max_size > b->widget.h ) {
-        g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Widgets to large (height) for box: %d %d", b->max_size, b->widget.h );
+    if ( active_widgets > 0 ) {
+        b->max_size += ( active_widgets - 1 ) * spacing;
+    }
+    if ( b->max_size > rem_height ) {
+        b->max_size = rem_height;
+        g_debug ( "Widgets to large (height) for box: %d %d", b->max_size, b->widget.h );
         return;
     }
     if ( active_widgets > 0 ) {
-        int    bottom = b->widget.h;
-        int    top    = 0;
-        double rem    = b->widget.h - b->max_size;
-        int    index  = 0;
+        int    top   = widget_padding_get_top ( WIDGET ( b ) );
+        double rem   = rem_height - b->max_size;
+        int    index = 0;
         for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
             widget * child = (widget *) iter->data;
             if ( child->enabled == FALSE ) {
@@ -80,40 +169,37 @@ static void vert_calculate_size ( box *b )
             if ( child->expand == TRUE ) {
                 // Re-calculate to avoid round issues leaving one pixel left.
                 int expanding_widgets_size = ( rem ) / ( expanding_widgets - index );
-                if ( child->end ) {
-                    bottom -= expanding_widgets_size;
-                    widget_move ( child, child->x, bottom );
-                    widget_resize ( child, b->widget.w, expanding_widgets_size );
-                    bottom -= b->padding;
-                }
-                else {
-                    widget_move ( child, child->x, top );
-                    top += expanding_widgets_size;
-                    widget_resize ( child, b->widget.w, expanding_widgets_size );
-                    top += b->padding;
-                }
+                widget_move ( child, widget_padding_get_left ( WIDGET ( b ) ), top );
+                top += expanding_widgets_size;
+                widget_resize ( child, rem_width, expanding_widgets_size );
+                top += spacing;
                 rem -= expanding_widgets_size;
                 index++;
             }
-            else if ( child->end ) {
-                bottom -= widget_get_height (  child );
-                widget_move ( child, child->x, bottom );
-                widget_resize ( child, b->widget.w, child->h );
-                bottom -= b->padding;
-            }
             else {
-                widget_move ( child, child->x, top );
+                widget_move ( child, widget_padding_get_left ( WIDGET ( b ) ), top );
                 top += widget_get_height (  child );
-                widget_resize ( child, b->widget.w, child->h );
-                top += b->padding;
+                top += spacing;
             }
         }
     }
+    b->max_size += widget_padding_get_padding_height ( WIDGET ( b ) );
 }
 static void hori_calculate_size ( box *b )
 {
+    int spacing           = distance_get_pixel ( b->spacing, ROFI_ORIENTATION_HORIZONTAL );
     int expanding_widgets = 0;
     int active_widgets    = 0;
+    int rem_width         = widget_padding_get_remaining_width ( WIDGET ( b ) );
+    int rem_height        = widget_padding_get_remaining_height ( WIDGET ( b ) );
+    for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
+        widget * child = (widget *) iter->data;
+        if ( child->enabled && child->expand == FALSE ) {
+            widget_resize ( child,
+                            widget_get_desired_width ( child ), //child->w,
+                            rem_height );
+        }
+    }
     b->max_size = 0;
     for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
         widget * child = (widget *) iter->data;
@@ -126,17 +212,19 @@ static void hori_calculate_size ( box *b )
             continue;
         }
         // Size used by fixed width widgets.
-        b->max_size += child->w;
+        if ( child->h > 0 ) {
+            b->max_size += child->w;
+        }
     }
-    b->max_size += MAX ( 0, ( ( active_widgets - 1 ) * b->padding ) );
-    if ( b->max_size > b->widget.w ) {
-        g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Widgets to large (width) for box: %d %d", b->max_size, b->widget.w );
+    b->max_size += MAX ( 0, ( ( active_widgets - 1 ) * spacing ) );
+    if ( b->max_size > ( rem_width ) ) {
+        b->max_size = rem_width;
+        g_debug ( "Widgets to large (width) for box: %d %d", b->max_size, b->widget.w );
         return;
     }
     if ( active_widgets > 0 ) {
-        int    right = b->widget.w;
-        int    left  = 0;
-        double rem   = b->widget.w - b->max_size;
+        int    left  = widget_padding_get_left ( WIDGET ( b ) );
+        double rem   = rem_width - b->max_size;
         int    index = 0;
         for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
             widget * child = (widget *) iter->data;
@@ -146,52 +234,30 @@ static void hori_calculate_size ( box *b )
             if ( child->expand == TRUE ) {
                 // Re-calculate to avoid round issues leaving one pixel left.
                 int expanding_widgets_size = ( rem ) / ( expanding_widgets - index );
-                if ( child->end ) {
-                    right -= expanding_widgets_size;
-                    widget_move ( child, right, child->y );
-                    widget_resize ( child, expanding_widgets_size, b->widget.h );
-                    right -= b->padding;
-                }
-                else {
-                    widget_move ( child, left, child->y );
-                    left += expanding_widgets_size;
-                    widget_resize ( child, expanding_widgets_size, b->widget.h );
-                    left += b->padding;
-                }
-                rem -= expanding_widgets_size;
+                widget_move ( child, left, widget_padding_get_top ( WIDGET ( b ) ) );
+                left += expanding_widgets_size;
+                widget_resize ( child, expanding_widgets_size, rem_height );
+                left += spacing;
+                rem  -= expanding_widgets_size;
                 index++;
             }
-            else if ( child->end ) {
-                right -= widget_get_width (  child );
-                widget_move ( child, right, child->y );
-                widget_resize ( child, child->w, b->widget.h );
-                right -= b->padding;
-            }
             else {
-                widget_move ( child, left, child->y );
+                widget_move ( child, left, widget_padding_get_top ( WIDGET ( b ) ) );
                 left += widget_get_width (  child );
-                widget_resize ( child, child->w, b->widget.h );
-                left += b->padding;
+                left += spacing;
             }
         }
     }
+    b->max_size += widget_padding_get_padding_width ( WIDGET ( b ) );
 }
 
 static void box_draw ( widget *wid, cairo_t *draw )
 {
     box *b = (box *) wid;
-    // Store current state.
-    cairo_save ( draw );
-    // Define a clipmask so we won't draw outside out widget.
-    cairo_rectangle ( draw, wid->x, wid->y, wid->w, wid->h );
-    cairo_clip ( draw );
-    // Set new x/y possition.
-    cairo_translate ( draw, wid->x, wid->y );
     for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
         widget * child = (widget *) iter->data;
         widget_draw ( child, draw );
     }
-    cairo_restore ( draw );
 }
 
 static void box_free ( widget *wid )
@@ -206,14 +272,24 @@ static void box_free ( widget *wid )
     g_free ( b );
 }
 
-void box_add ( box *box, widget *child, gboolean expand, gboolean end )
+void box_add ( box *box, widget *child, gboolean expand )
 {
     if ( box == NULL ) {
         return;
     }
-    child->expand = expand;
-    child->end    = end;
-    child->parent = WIDGET ( box );
+    // Make sure box is width/heigh enough.
+    if ( box->type == ROFI_ORIENTATION_VERTICAL ) {
+        int width = box->widget.w;
+        width         = MAX ( width, child->w + widget_padding_get_padding_width ( WIDGET ( box ) ) );
+        box->widget.w = width;
+    }
+    else {
+        int height = box->widget.h;
+        height        = MAX ( height, child->h + widget_padding_get_padding_height ( WIDGET ( box ) ) );
+        box->widget.h = height;
+    }
+    child->expand = rofi_theme_get_boolean ( child, "expand", expand );
+    g_assert ( child->parent == WIDGET ( box ) );
     box->children = g_list_append ( box->children, (void *) child );
     widget_update ( WIDGET ( box ) );
 }
@@ -228,7 +304,7 @@ static void box_resize ( widget *widget, short w, short h )
     }
 }
 
-static gboolean box_clicked ( widget *wid, xcb_button_press_event_t *xbe, G_GNUC_UNUSED void *udata )
+static widget *box_find_mouse_target ( widget *wid, WidgetType type, gint x, gint y )
 {
     box *b = (box *) wid;
     for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
@@ -236,49 +312,35 @@ static gboolean box_clicked ( widget *wid, xcb_button_press_event_t *xbe, G_GNUC
         if ( !child->enabled ) {
             continue;
         }
-        if ( widget_intersect ( child, xbe->event_x, xbe->event_y ) ) {
-            xcb_button_press_event_t rel = *xbe;
-            rel.event_x -= child->x;
-            rel.event_y -= child->y;
-            return widget_clicked ( child, &rel );
+        if ( widget_intersect ( child, x, y ) ) {
+            gint   rx      = x - child->x;
+            gint   ry      = y - child->y;
+            widget *target = widget_find_mouse_target ( child, type, rx, ry );
+            if ( target != NULL ) {
+                return target;
+            }
         }
     }
-    return FALSE;
-}
-static gboolean box_motion_notify ( widget *wid, xcb_motion_notify_event_t *xme )
-{
-    box *b = (box *) wid;
-    for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
-        widget * child = (widget *) iter->data;
-        if ( !child->enabled ) {
-            continue;
-        }
-        if ( widget_intersect ( child, xme->event_x, xme->event_y ) ) {
-            xcb_motion_notify_event_t rel = *xme;
-            rel.event_x -= child->x;
-            rel.event_y -= child->y;
-            return widget_motion_notify ( child, &rel );
-        }
-    }
-    return FALSE;
+    return NULL;
 }
 
-box * box_create ( boxType type, short x, short y, short w, short h )
+box * box_create ( widget *parent, const char *name, RofiOrientation type )
 {
     box *b = g_malloc0 ( sizeof ( box ) );
-    b->type                 = type;
-    b->widget.x             = x;
-    b->widget.y             = y;
-    b->widget.w             = w;
-    b->widget.h             = h;
-    b->widget.draw          = box_draw;
-    b->widget.free          = box_free;
-    b->widget.resize        = box_resize;
-    b->widget.update        = box_update;
-    b->widget.clicked       = box_clicked;
-    b->widget.motion_notify = box_motion_notify;
-    b->widget.enabled       = TRUE;
+    // Initialize widget.
+    widget_init ( WIDGET ( b ), parent, WIDGET_TYPE_UNKNOWN, name );
+    b->type                      = type;
+    b->widget.draw               = box_draw;
+    b->widget.free               = box_free;
+    b->widget.resize             = box_resize;
+    b->widget.update             = box_update;
+    b->widget.find_mouse_target  = box_find_mouse_target;
+    b->widget.get_desired_height = box_get_desired_height;
+    b->widget.get_desired_width  = box_get_desired_width;
 
+    b->type = rofi_theme_get_orientation ( WIDGET ( b ), "orientation", b->type );
+
+    b->spacing = rofi_theme_get_distance ( WIDGET ( b ), "spacing", DEFAULT_SPACING );
     return b;
 }
 
@@ -287,26 +349,11 @@ static void box_update ( widget *wid  )
     box *b = (box *) wid;
     switch ( b->type )
     {
-    case BOX_VERTICAL:
+    case ROFI_ORIENTATION_VERTICAL:
         vert_calculate_size ( b );
         break;
-    case BOX_HORIZONTAL:
+    case ROFI_ORIENTATION_HORIZONTAL:
     default:
         hori_calculate_size ( b );
     }
 }
-int box_get_fixed_pixels ( box *box )
-{
-    if ( box != NULL ) {
-        return box->max_size;
-    }
-    return 0;
-}
-
-void box_set_padding ( box * box, unsigned int padding )
-{
-    if ( box != NULL ) {
-        box->padding = padding;
-        widget_queue_redraw ( WIDGET ( box ) );
-    }
-}
